@@ -66,6 +66,20 @@ typedef struct txg_list {
 	txg_node_t	*tl_head[TXG_SIZE];
 } txg_list_t;
 
+/*
+ * Wait flags for txg_wait_synced_flags(). By default (TXG_WAIT_NONE), it will
+ * wait until the wanted txg is reached, or block forever. Additional flags
+ * indicate other conditions that the caller is interested in, that will cause
+ * the wait to break and return an error code describing the condition.
+ */
+typedef enum {
+	/* No special flags. Guaranteed to block forever or return 0 */
+	TXG_WAIT_NONE   = 0,
+
+	/* If a signal arrives while waiting, abort and return EINTR */
+	TXG_WAIT_SIGNAL = (1 << 0),
+} txg_wait_flag_t;
+
 struct dsl_pool;
 
 extern void txg_init(struct dsl_pool *dp, uint64_t txg);
@@ -86,13 +100,16 @@ extern void txg_kick(struct dsl_pool *dp, uint64_t txg);
  * Try to make this happen as soon as possible (eg. kick off any
  * necessary syncs immediately).  If txg==0, wait for the currently open
  * txg to finish syncing.
+ * See txg_wait_flag_t above for a description of how the flags affect the wait.
  */
-extern void txg_wait_synced(struct dsl_pool *dp, uint64_t txg);
+extern int txg_wait_synced_flags(struct dsl_pool *dp, uint64_t txg,
+    txg_wait_flag_t flags);
 
 /*
- * Wait as above. Returns true if the thread was signaled while waiting.
+ * Traditional form of txg_wait_synced_flags, waits forever.
+ * Shorthand for VERIFY0(txg_wait_synced_flags(dp, TXG_WAIT_NONE))
  */
-extern boolean_t txg_wait_synced_sig(struct dsl_pool *dp, uint64_t txg);
+extern void txg_wait_synced(struct dsl_pool *dp, uint64_t txg);
 
 /*
  * Wait until the given transaction group, or one after it, is
