@@ -393,5 +393,5 @@ dmu_write_uio_direct(dnode_t *dn, zfs_uio_t *uio, uint64_t size, dmu_tx_t *tx)
 }
 #endif /* _KERNEL */
 
-EXPORT_SYMBOL(dmu_read_uio_direct);
-EXPORT_SYMBOL(dmu_write_uio_direct);
+EXPORT_SYMBOL(dmu_read_abd);
+EXPORT_SYMBOL(dmu_write_abd);
