@@ -40,4 +40,4 @@ if [ x"${OUT}" != x ]; then
     echo
 fi
 
-exit $RET
+exit "$RET"
